package ink.wgink.module.activiti.service.oa.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.module.activiti.enums.oa.SelectAssigneeTypeEnum;
import ink.wgink.module.activiti.enums.oa.field.FieldEditRecordEnum;
import ink.wgink.module.activiti.pojo.bos.oa.OaFormReportUnWantedValueBO;
import ink.wgink.module.activiti.pojo.vos.oa.OaFormReportRecordFieldVO;
import ink.wgink.module.activiti.service.activiti.IActivitiModelService;
import ink.wgink.module.activiti.service.oa.IOaFormReportService;
import ink.wgink.module.form.service.design.IFormFieldService;
import ink.wgink.module.form.service.report.IFormReportService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.app.AppTokenUser;
import ink.wgink.pojo.bos.UserInfoBO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.util.date.DateUtil;
import org.activiti.engine.HistoryService;
import org.activiti.engine.IdentityService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.history.HistoricProcessInstance;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: OaFormReportServiceImpl
 * @Description: oa表单上报
 * @Author: wanggeng
 * @Date: 2022/3/22 22:05
 * @Version: 1.0
 */
@Service
public class OaFormReportServiceImpl extends DefaultBaseService implements IOaFormReportService {

    @Autowired
    private IFormFieldService formFieldService;
    @Autowired
    private IFormReportService formReportService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private HistoryService historyService;
    @Autowired
    private TaskService taskService;
    @Autowired
    private IdentityService identityService;

    @Override
    public void save(String processDefinitionId, String formCode, Integer formVersion, Map<String, Object> params) {
        UserInfoBO currentUser = securityComponent.getCurrentUser();
        saveByUserIdAndUserName(currentUser.getUserId(), currentUser.getUserName(), processDefinitionId, formCode, formVersion, params);
    }

    @Override
    public void saveByToken(String token, String processDefinitionId, String formCode, Integer formVersion, Map<String, Object> params) {
        AppTokenUser appTokenUser = getAppTokenUser(token);
        saveByUserIdAndUserName(appTokenUser.getId(), appTokenUser.getName(), processDefinitionId, formCode, formVersion, params);
    }

    @Override
    public void saveByUserIdAndUserName(String userId, String userName, String processDefinitionId, String formCode, Integer formVersion, Map<String, Object> params) {
        OaFormReportUnWantedValueBO oaFormReportUnWantedValueBO = getOaFormReportUnWantedValue(params);

        LOG.debug("保存表单");
        String uid = formReportService.saveAndReturnId(formCode, formVersion, params);
        LOG.debug("保存附件");
        LOG.debug("保存");
        LOG.debug("定义流程发起人");
        Map<String, Object> variables = getHashMap(2);
        variables.put(KEY_FORM_CODE, formCode);
        variables.put(KEY_FORM_VERSION, formVersion);
        variables.put(KEY_REPORT_UID, uid);
        variables.put(KEY_START_ASSIGNEE_USER, userId);
        identityService.setAuthenticatedUserId(userId);
        LOG.debug("发起流程");
        ProcessInstance processInstance = runtimeService.startProcessInstanceById(processDefinitionId, variables);
        LOG.debug("更新表单数据的流程实例ID");
        formReportService.updateProcessInstanceId(formCode, formVersion, uid, processInstance.getProcessInstanceId());
        LOG.debug("执行第一个节点");
        Task task = taskService.createTaskQuery().processInstanceId(processInstance.getId()).taskCandidateOrAssigned(userId).singleResult();
        if (task == null) {
            throw new SearchException("任务不存在");
        }
        // 网关条件
        if (!StringUtils.isBlank(oaFormReportUnWantedValueBO.getExc())) {
            params.put(IActivitiModelService.EXCLUSIVE_GATEWAY_CONDITION_KEY, oaFormReportUnWantedValueBO.getExc());
        }
        if (!oaFormReportUnWantedValueBO.getNextEndEvent()) {
            setFormReportAssignee(oaFormReportUnWantedValueBO.getSelectType(), oaFormReportUnWantedValueBO.getAssignees(), params);
        }
        setRecordFields(task.getId(), oaFormReportUnWantedValueBO.getRecordFields(), userId, userName, params);
        taskService.setVariablesLocal(task.getId(), params);
        taskService.complete(task.getId(), params);
    }

    @Override
    public void update(String taskId, String formCode, Integer formVersion, Integer isNeedClaim, String uid, Map<String, Object> params) {
        UserInfoBO currentUser = securityComponent.getCurrentUser();
        updateByUserIdAndUserName(currentUser.getUserId(), currentUser.getUserName(), taskId, formCode, formVersion, isNeedClaim, uid, params);
    }

    @Override
    public void updateByToken(String token, String taskId, String formCode, Integer formVersion, Integer isNeedClaim, String uid, Map<String, Object> params) {
        AppTokenUser appTokenUser = getAppTokenUser(token);
        updateByUserIdAndUserName(appTokenUser.getId(), appTokenUser.getName(), taskId, formCode, formVersion, isNeedClaim, uid, params);
    }

    @Override
    public void updateByUserIdAndUserName(String userId, String userName, String taskId, String formCode, Integer formVersion, Integer isNeedClaim, String uid, Map<String, Object> params) {
        OaFormReportUnWantedValueBO oaFormReportUnWantedValueBO = getOaFormReportUnWantedValue(params);
        formReportService.update(formCode, formVersion, uid, params);
        if (isNeedClaim == 1) {
            LOG.debug("认领任务");
            taskService.claim(taskId, userId);
        }
        // 网关条件
        if (!StringUtils.isBlank(oaFormReportUnWantedValueBO.getExc())) {
            params.put(IActivitiModelService.EXCLUSIVE_GATEWAY_CONDITION_KEY, oaFormReportUnWantedValueBO.getExc());
        }
        if (!oaFormReportUnWantedValueBO.getNextEndEvent()) {
            setFormReportAssignee(oaFormReportUnWantedValueBO.getSelectType(), oaFormReportUnWantedValueBO.getAssignees(), params);
        }
        setRecordFields(taskId, oaFormReportUnWantedValueBO.getRecordFields(), userId, userName, params);
        taskService.setVariablesLocal(taskId, params);
        taskService.complete(taskId, params);
    }

    /**
     * 保存记录的字段列表
     *
     * @param taskId
     * @param oaFormReportRecordFieldVOs
     * @param userId
     * @param userName
     * @param reportParams
     */
    private void setRecordFields(String taskId, List<OaFormReportRecordFieldVO> oaFormReportRecordFieldVOs, String userId, String userName, Map<String, Object> reportParams) {
        if (oaFormReportRecordFieldVOs.isEmpty()) {
            return;
        }
        String currentTime = DateUtil.getTime();
        Map<String, Object> processVariables = taskService.getVariables(taskId);
        for (OaFormReportRecordFieldVO oaFormReportRecordFieldVO : oaFormReportRecordFieldVOs) {
            Object reportRecordObject = reportParams.get(oaFormReportRecordFieldVO.getFieldName());
            String reportRecordValue = reportRecordObject == null ? "" : reportRecordObject.toString();

            // 任务中存储的签批变量关键字
            String taskVariableKey = EDIT_RECORD_KEY_PREFIX + oaFormReportRecordFieldVO.getFieldExplain();
            JSONObject taskSignJsonObject = new JSONObject();
            taskSignJsonObject.put("userId", userId);
            taskSignJsonObject.put("userName", userName);
            taskSignJsonObject.put("content", reportRecordValue);
            taskSignJsonObject.put("time", currentTime);
            reportParams.put(taskVariableKey, taskSignJsonObject.toJSONString());
            if (StringUtils.equals(oaFormReportRecordFieldVO.getEditHistory(), FieldEditRecordEnum.LATEST.getValue())) {
                // 审批存储在流程变量中
                continue;
            }
            if (StringUtils.equals(oaFormReportRecordFieldVO.getEditHistory(), FieldEditRecordEnum.ALL.getValue())) {
                // 流程中存储的签批变量关键字
                String processSignKey = String.format("%s%s:%s", EDIT_RECORD_KEY_PREFIX, oaFormReportRecordFieldVO.getFieldName(), oaFormReportRecordFieldVO.getFieldExplain());
                // 合并后的会签存储在流程变量中
                Object processSignValueObj = processVariables.get(processSignKey);
                JSONArray processSignValueJsonArray;
                if (processSignValueObj != null) {
                    processSignValueJsonArray = JSONArray.parseArray(processSignValueObj.toString());
                } else {
                    processSignValueJsonArray = new JSONArray();
                }
                processSignValueJsonArray.add(taskSignJsonObject);
                taskService.setVariable(taskId, processSignKey, processSignValueJsonArray.toJSONString());
                continue;
            }
            throw new ParamsException("记录类型错误");
        }
    }

    /**
     * 获取上报表单不需要的值
     *
     * @param reportParams
     * @return
     */
    private OaFormReportUnWantedValueBO getOaFormReportUnWantedValue(Map<String, Object> reportParams) {
        List<String> assignees = (List<String>) reportParams.get(IOaFormReportService.KEY_ASSIGNEES);
        String selectType = reportParams.get(IOaFormReportService.KEY_SELECT_TYPE).toString();
        Boolean isNextEndEvent = Boolean.parseBoolean(reportParams.get(IOaFormReportService.KEY_IS_NEXT_END_EVENT).toString());
        Object excObj = reportParams.get(IActivitiModelService.EXCLUSIVE_GATEWAY_CONDITION_KEY);
        List<OaFormReportRecordFieldVO> oaFormReportRecordFieldVOs = JSONArray.parseArray(JSON.toJSONString(reportParams.get(IOaFormReportService.KEY_RECORD_FIELDS)), OaFormReportRecordFieldVO.class);
        // 移除无效的内容
        reportParams.remove(IOaFormReportService.KEY_ASSIGNEES);
        reportParams.remove(IOaFormReportService.KEY_SELECT_TYPE);
        reportParams.remove(IOaFormReportService.KEY_IS_NEXT_END_EVENT);
        reportParams.remove(IOaFormReportService.KEY_RECORD_FIELDS);
        reportParams.remove(IActivitiModelService.EXCLUSIVE_GATEWAY_CONDITION_KEY);
        // 归类到对象
        OaFormReportUnWantedValueBO oaFormReportUnWantedValueBO = new OaFormReportUnWantedValueBO();
        oaFormReportUnWantedValueBO.setAssignees(assignees);
        oaFormReportUnWantedValueBO.setSelectType(selectType);
        oaFormReportUnWantedValueBO.setNextEndEvent(isNextEndEvent);
        oaFormReportUnWantedValueBO.setExc(excObj == null ? "" : excObj.toString());
        oaFormReportUnWantedValueBO.setRecordFields(oaFormReportRecordFieldVOs);
        return oaFormReportUnWantedValueBO;
    }

    /**
     * 设置表单提交代理人
     *
     * @param selectType       选择类型
     * @param assignees        代理人列表
     * @param formReportParams 表单参数
     */
    private void setFormReportAssignee(String selectType, List<String> assignees, Map<String, Object> formReportParams) {
        if (StringUtils.equals(selectType, SelectAssigneeTypeEnum.CHECKBOX.getValue())) {
            formReportParams.put(IActivitiModelService.ASSIGNEE_LIST, assignees);
        } else {
            formReportParams.put(IOaFormReportService.KEY_ASSIGNEE_USER, assignees.get(0));
        }
    }

    @Override
    public Map<String, Object> get(String formCode, Integer formVersion, String uid) {
        Map<String, Object> detail = formReportService.get(formCode, formVersion, uid);
        // 查询会签列表
        String processInstanceId = detail.get("process_instance_id").toString();
        HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery().processInstanceId(processInstanceId).includeProcessVariables().singleResult();
        Map<String, Object> processVariablesMap = historicProcessInstance.getProcessVariables();
        for (Map.Entry<String, Object> entry : processVariablesMap.entrySet()) {
            String key = entry.getKey();
            if (!key.startsWith(IOaFormReportService.EDIT_RECORD_KEY_PREFIX)) {
                continue;
            }
            Object valueObj = entry.getValue();
            if (valueObj == null) {
                continue;
            }
            String value = valueObj.toString();
            if (StringUtils.isBlank(value)) {
                continue;
            }
            String[] keyArray = key.split(":");
            // 普通签批不处理，普通签批格式(RECORD:fieldName)，会签格式(RECORD:fieldName:fieldExplain)
            if (keyArray.length == 2) {
                continue;
            }
            String fieldName = keyArray[1];
            detail.put("_sign_" + fieldName, JSONArray.parseArray(value));
        }
        return detail;
    }

    @Override
    public Map<String, Object> getPO(String formCode, Integer formVersion, String uid) {
        return formReportService.get(formCode, formVersion, uid);
    }

    @Override
    public List<Map<String, Object>> list(String formCode, Integer formVersion, Map<String, Object> params) {
        return formReportService.list(formCode, formVersion, params);
    }

    @Override
    public List<Map<String, Object>> list(String formCode, Integer formVersion, List<String> uids) {
        Map<String, Object> params = getHashMap(6);
        params.put("uids", uids);
        return formReportService.list(formCode, formVersion, params);
    }

    @Override
    public SuccessResultList<List<Map<String, Object>>> listPage(String formCode, Integer formVersion, ListPage page) {
        return formReportService.listPage(formCode, formVersion, page);
    }
}
