function OaFormUtil(layui) {
    var $ = layui.$;
    var win = $(window);
    var layer = layui.layer;
    var restAjax = layui.restajax;
    var processImageEnlargeScale = 0;

    /**
     * 设置保存历史字段列表
     * @param formData
     * @param fields
     */
    this.setEditHistoryFields = function (formData, fields) {
        var recordFields = [];
        for (var i = 0, field; field = fields[i++];) {
            if (field.editHistory === 'latest' || field.editHistory === 'all') {
                recordFields.push({
                    fieldName: field.fieldName,
                    fieldExplain: $('#' + field.fieldName + 'Box').find('label').text().replace(/[\*\:]/g, ''),
                    editHistory: field.editHistory
                });
            }
        }
        formData.field.recordFields = recordFields;
    }

    /**
     * 设置签字列表
     * @param formData
     */
    this.initSignList = function (formData) {
        for (var key in formData) {
            if (key.indexOf('_sign_') != 0) {
                continue;
            }
            var signPreDomKey = key.substring(6, key.length);
            var signList = formData[key];
            var signListDom = '<div><span>记录</span>';
            for (var i = 0, item; item = signList[i++];) {
                signListDom += [
                    '<div>',
                    '   <span>' + item.userName + '：</span>',
                    '   <span>' + item.content + ' - </span>',
                    '   <span>' + item.time + '</span>',
                    '</div>'
                ].join('');
            }
            signListDom += '</div>';
            $('#' + signPreDomKey + 'Block').append(signListDom);
            delete formData[key];
            delete formData[signPreDomKey];
        }
    }

    /**
     * 初始化文本框签字列表
     * @param formData
     */
    this.initTextareaSignList = function (formData) {
        for (var key in formData) {
            if (key.indexOf('_sign_') != 0) {
                continue;
            }
            var signPreDomKey = key.substring(6, key.length);
            var signList = formData[key];
            var signListText = '';
            for (var i = 0, item; item = signList[i++];) {
                signListText += item.userName + '：' + item.content + ' - ' + item.time +'\n';
            }
            formData[signPreDomKey] = signListText;
            delete formData[key];
        }
    }

    /**
     * 初始化日志
     */
    this.initProcessLog = function (processInstanceId) {
        restAjax.get(restAjax.path('api/oa/list-process-log/process-instance-id/{processInstanceId}', [processInstanceId]), {}, null, function (code, data) {
            var ul = '<ul class="layui-timeline">';
            for (var i = 0, item; item = data[i++];) {
                // 处理签批内容
                var taskVariables = item.taskVariables;
                var signDom = '';
                for (var key in taskVariables) {
                    if (key.indexOf('RECORD:') != 0) {
                        continue;
                    }
                    var sign = JSON.parse(taskVariables[key]);
                    signDom += '<div class="log-sign-box">' +
                        '<span class="log-sign title" style="font-weight: bold;">【' + key.replace('RECORD:', '') + '】</span>' +
                        '<span class="log-sign content">内容：' + sign.content + '</span>' +
                        '<span class="log-sign time">时间：' + sign.time + '</span>' +
                        '</div>';
                }

                var li = [
                    '<li class="layui-timeline-item">',
                    '   <i class="layui-icon layui-timeline-axis">&#xe63f;</i>',
                    '   <div class="layui-timeline-content layui-text">',
                    '       <h3 class="layui-timeline-title">' + item.startTime + '</h3>',
                    '       <table class="layui-table">',
                    '           <colgroup>',
                    '               <col width="100">',
                    '               <col>',
                    '           </colgroup>',
                    '           <tbody>',
                    '           <tr>',
                    '               <td class="row-title">任务名称</td>',
                    '               <td>' + item.taskName + '</td>',
                    '           </tr>',
                    '           <tr>',
                    '               <td class="row-title">节点状态</td>',
                    '               <td>' + (item.taskStatus == 'alreadyDone' ? '已办' : '待办') + '</td>',
                    '           </tr>',
                    '           <tr>',
                    '               <td class="row-title">结束时间</td>',
                    '               <td>' + item.endTime + '</td>',
                    '           </tr>',
                    '           <tr>',
                    '               <td class="row-title">耗时</td>',
                    '               <td>' + item.usedTime + '</td>',
                    '           </tr>',
                    '           <tr>',
                    '               <td class="row-title">处理人</td>',
                    '               <td>' + item.userNames + '</td>',
                    '           </tr>',
                    '           <tr>',
                    '               <td class="row-title">签批</td>',
                    '               <td>',
                    signDom,
                    '               </td>',
                    '           </tr>',
                    '           </tbody>',
                    '       </table>',
                    '   </div>',
                    '</li>'
                ].join('');
                ul += li;
            }
            ul += '</ul>';
            $('#processLog').append(ul);
        }, function (code, data) {
            layer.msg(data.msg);
        });
    }

    /**
     * 初始化流程图
     * @param processInstanceId
     */
    this.initProcessImage = function (processInstanceId) {
        function initRuntimeProcessImage() {
            $('#runtimeProcessImage').attr('src', 'route/activiti/model/get-runtime-process-image/' + processInstanceId);
        }

        initRuntimeProcessImage();

        function setProcessImageSize() {
            $('#runtimeProcessImage').css('height', 100 + processImageEnlargeScale + '%');
        }

        $('#searchPlus').click(function () {
            processImageEnlargeScale += 10;
            setProcessImageSize();
        })
        $('#searchMinus').click(function () {
            processImageEnlargeScale -= 10;
            setProcessImageSize();
        })
        $('#searchDefault').click(function () {
            processImageEnlargeScale = 0;
            setProcessImageSize();
        })
    }

    /**
     * 初始化选择项
     */
    this.initTabItem = function () {
        $('.layui-tab-item, #runtimeProcessImageBox').css({
            height: (win.height() - 138) + 'px',
            overflow: 'auto'
        });
    }

    /**
     * 初始化提交按钮
     * @param confirmAssignees
     */
    this.initSubmitBtns = function (confirmAssignees) {
        var btnColor = ['', 'layui-btn-normal', 'layui-btn-warm'];
        var btns = '';
        for (var i = 0, item; item = confirmAssignees[i++];) {
            btns += '<button type="button" id="submitFormBtn' + i + '" class="layui-btn layui-btn-sm ' + btnColor[(i - 1) % 3] + ' confirm-btn" lay-submit lay-filter="submit' + i + '" data-select-type="' + item.nodeType + '" data-index="' + (i - 1) + '" data-next-end-event="' + item.nextEndEvent + '" data-btn-exc="' + item.btnExc + '">' + item.btnText + '</button>'
        }
        $('#submitBtnGroup').append(btns);
    }

    /**
     * @description 禁止字段操作
     * @param fields {Array} [{fieldName:'', isEditable: 0|1}]
     */
    this.disableFields = function (fields) {
        for (var i = 0, item; item = fields[i++];) {
            if (parseInt(item.isEditable) === 0) {
                var boxId = '#' + item.fieldName + 'Box';
                $(boxId).css('position', 'relative');
                $(boxId).append('<div style="width: 100%; height: 100%; background-color: #fff0; position: absolute; top: 0; left: 0; cursor: not-allowed;"></div>');
                var doms = $(boxId).find('input,button,textarea');
                doms.attr('disabled', 'disabled');
                doms.attr('lay-verify', '');
            }
        }
    }

    /**
     * 禁用表单全部字段
     */
    this.disableFormAllFields = function () {
        var doms = $('#reportForm').find('input,button,textarea');
        doms.attr('disabled', 'disabled');
        doms.attr('lay-verify', '');
    }

    /**
     * @description 隐藏字段操作
     * @param fields {Array} [{fieldName:'', isVisible: 0|1}]
     */
    this.hideFields = function (fields) {
        for (var i = 0, item; item = fields[i++];) {
            if (parseInt(item.isVisible) === 0) {
                var boxId = '#' + item.fieldName + 'Box';
                $(boxId).hide();
            }
        }
    }

}